@@ -8,7 +8,7 @@
 //! It can multiple accessing data of multiple MumbleLinks, and allows multiple clients
 //! to request the data.
 
-use std::{fmt::Debug, path::PathBuf};
+use std::path::PathBuf;
 
 use log::LevelFilter;
 
@@ -43,25 +43,3 @@ pub fn log_init(
     ])?;
     Ok(())
 }
-/*
-Ctypes -> clink, context,
-            from, from
-usefulT -> umlink (umctx)
-
-CacheT -> LinkCache
-
-Multiple -> MlMap
-
-Concurrent -> Mutex, Arc
-
-udp -> async, sync
-grpc -> async
-
-*/
-fn log_error<T>(e: T) -> T
-where
-    T: Debug,
-{
-    log::error!("{:?}", &e);
-    e
-}
