@@ -1,3 +1,8 @@
+use std::net::Ipv4Addr;
+
+use bitflags::bitflags;
+use num_derive::FromPrimitive;
+use num_derive::ToPrimitive;
 use serde::{Deserialize, Serialize};
 
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
@@ -10,65 +15,6 @@ pub struct MumbleLink {
     pub identity: CIdentity,
     pub context: CMumbleContext,
 }
-#[derive(Deserialize, Clone, Debug, Default, Serialize)]
-pub struct MumbleIdentity {
-    pub name: String,
-    pub profession: u32,
-    pub spec: u32,
-
-    pub race: u32,
-
-    pub map_id: u32,
-
-    pub world_id: u32,
-
-    pub team_color_id: u32,
-
-    pub commander: bool,
-
-    pub fov: f32,
-
-    pub uisz: u32,
-}
-#[derive(Copy, Clone, Debug, Default, Deserialize, Serialize)]
-pub struct MumbleContext {
-    pub server_address: [u8; 4],
-
-    pub map_id: u32,
-
-    pub map_type: u32,
-
-    pub shard_id: u32,
-
-    pub instance: u32,
-
-    pub build_id: u32,
-    ///Bitmask: Bit 1 = IsMapOpen, Bit 2 = IsCompassTopRight,
-    ///Bit 3 = DoesCompassHaveRotationEnabled, Bit 4 = Game
-    ///has focus, Bit 5 = Is in Competitive game mode, Bit 6
-    ///= Textbox has focus, Bit 7 = Is in Combat
-    pub ui_state: u32,
-
-    pub compass_width: u32,
-
-    pub compass_height: u32,
-
-    pub compass_rotation: f32,
-
-    pub player_x: f32,
-
-    pub player_y: f32,
-
-    pub map_center_x: f32,
-
-    pub map_center_y: f32,
-
-    pub map_scale: f32,
-
-    pub process_id: u32,
-
-    pub mount_index: u32,
-}
 
 impl MumbleLink {
     /// The most used function probably. will check if the `link_ptr->ui_tick > &self.ui_tick`
@@ -103,6 +49,122 @@ impl MumbleLink {
     }
 }
 
+#[derive(Debug, Default, Clone, Copy, Serialize, Deserialize)]
+#[repr(C)]
+/// The mumble context as stored inside the context field of CMumbleLink.
+/// the first 48 bytes Mumble uses for identification is upto build_id field
+/// the rest of the fields after build_id are provided by gw2 for addon devs.
+pub struct CMumbleContext {
+    /// first byte is `2` if ipv4. and `[4..7]` bytes contain the ipv4 octets.
+    pub server_address: [u8; 28], // contains sockaddr_in or sockaddr_in6
+    pub map_id: u32,
+    pub map_type: u32,
+    pub shard_id: u32,
+    pub instance: u32,
+    pub build_id: u32,
+    // Additional data that gw2 provides us
+    pub ui_state: u32, // Bitmask: Bit 1 = IsMapOpen, Bit 2 = IsCompassTopRight, Bit 3 = DoesCompassHaveRotationEnabled, Bit 4 = Game has focus, Bit 5 = Is in Competitive game mode, Bit 6 = Textbox has focus, Bit 7 = Is in Combat
+    pub compass_width: u16, // pixels
+    pub compass_height: u16, // pixels
+    pub compass_rotation: f32, // radians
+    pub player_x: f32, // continentCoords
+    pub player_y: f32, // continentCoords
+    pub map_center_x: f32, // continentCoords
+    pub map_center_y: f32, // continentCoords
+    pub map_scale: f32,
+    pub process_id: u32,
+    pub mount_index: u8,
+}
+#[derive(Debug, FromPrimitive, ToPrimitive, Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Ord)]
+pub enum Mount {
+    None = 0,
+    Jackal = 1,
+    Griffon = 2,
+    Springer = 3,
+    Skimmer = 4,
+    Raptor = 5,
+    RollerBeetle = 6,
+    Warclaw = 7,
+    Skyscale = 8,
+}
+
+bitflags! {
+    pub struct UIState: u32 {
+        const IS_MAP_OPEN = 0b00000001;
+        const IS_COMPASS_TOP_RIGHT = 0b00000010;
+        const DOES_COMPASS_HAVE_ROTATION_ENABLED = 0b00000100;
+        const GAME_HAS_FOCUS = 0b00001000;
+        const IN_COMPETITIVE_GAMEMODE = 0b00010000;
+        const TEXTBOX_FOCUS = 0b00100000;
+        const IS_IN_COMBAT = 0b01000000;
+    }
+}
+impl CMumbleContext {
+    pub fn ui_state(&self) -> UIState {
+        UIState::from_bits(self.ui_state).unwrap_or_else(|| {
+            log::error!(
+                "ui_state is in an invalid bitset state. ui_state uint32_t: {}",
+                self.ui_state
+            );
+            panic!("ui_state bitset parse failed")
+        })
+    }
+
+    pub fn update(&mut self, link_ptr: *const CMumbleLink) {
+        let mc = unsafe {
+            std::ptr::read_volatile(&(*link_ptr).context as *const u8 as *const CMumbleContext)
+        };
+        *self = mc;
+    }
+    /// first byte is `2` if ipv4. and `[4..7]` bytes contain the ipv4 octets.
+    /// contains sockaddr_in or sockaddr_in6
+    pub fn get_map_ip(&self) -> Ipv4Addr {
+        if self.server_address[0] != 2 {
+            log::error!(
+                "ip addr first byte is not 2. server_address byte array: {:?}",
+                self.server_address
+            );
+            panic!("ipaddr parsing failed for CMumble Context");
+        }
+
+        let ip = Ipv4Addr::from([
+            self.server_address[4],
+            self.server_address[5],
+            self.server_address[6],
+            self.server_address[7],
+        ]);
+        ip
+    }
+
+    pub fn get_mount(&self) -> Mount {
+        use num_traits::FromPrimitive;
+        Mount::from_u8(self.mount_index).unwrap_or_else(|| {
+            log::error!(
+                "invalid mount state in CMumbleContext. {}",
+                self.mount_index
+            );
+            panic!("mount_index parse failed")
+        })
+    }
+}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+/// The json structure of the Identity field inside Cmumblelink.
+/// the json string is null terminated and utf-16 encoded. so, need to use
+/// Widestring crate's U16Cstring to first parse the bytes and then, convert to
+/// String before deserializing to CIdentity
+pub struct CIdentity {
+    pub name: String,
+    pub profession: u32,
+    pub spec: u32,
+    pub race: u32,
+    pub map_id: u32,
+    pub world_id: u32,
+    pub team_color_id: u32,
+    pub commander: bool,
+    pub fov: f32,
+    pub uisz: u32,
+}
 impl CIdentity {
     pub fn update(&mut self, link_ptr: *const CMumbleLink) -> anyhow::Result<()> {
         use widestring::U16CStr;
@@ -131,42 +193,6 @@ impl CIdentity {
     }
 }
 
-impl MumbleContext {
-    pub fn update(&mut self, link_ptr: *const CMumbleLink) {
-        let mc = unsafe {
-            std::ptr::read_volatile(&(*link_ptr).context as *const u8 as *const CMumbleContext)
-        };
-        self.server_address
-            .copy_from_slice(&mc.server_address[4..7]);
-        self.map_id = mc.map_id;
-        self.map_type = mc.map_type;
-        self.shard_id = mc.shard_id;
-        self.instance = mc.instance;
-        self.build_id = mc.build_id;
-        self.ui_state = mc.ui_state;
-        self.compass_width = mc.compass_width as u32;
-        self.compass_height = mc.compass_height as u32;
-        self.compass_rotation = mc.compass_rotation;
-        self.player_x = mc.player_x;
-        self.player_y = mc.player_y;
-        self.map_center_x = mc.map_center_x;
-        self.map_center_y = mc.map_center_y;
-        self.map_scale = mc.map_scale;
-        self.process_id = mc.process_id;
-        self.mount_index = mc.mount_index as u32;
-    }
-}
-
-impl CMumbleContext {
-    pub fn update(&mut self, link_ptr: *const CMumbleLink) {
-        let mc = unsafe {
-            std::ptr::read_volatile(&(*link_ptr).context as *const u8 as *const CMumbleContext)
-        };
-        *self = mc;
-    }
-}
-
-// pub const MUMBLE_LINK_SIZE: usize = 1193;
 pub const C_MUMBLE_LINK_SIZE: usize = std::mem::size_of::<CMumbleLink>();
 pub const USEFUL_C_MUMBLE_LINK_SIZE: usize = 1193;
 
@@ -187,51 +213,6 @@ pub struct CMumbleLink {
     pub context: [u8; 256],
     pub description: [u8; 4096],
 }
-#[derive(Debug, Default, Clone, Copy, Serialize, Deserialize)]
-#[repr(C)]
-/// The mumble context as stored inside the context field of CMumbleLink.
-/// the first 48 bytes Mumble uses for identification is upto build_id field
-/// the rest of the fields after build_id are provided by gw2 for addon devs.
-pub struct CMumbleContext {
-    /// first byte is `2` if ipv4. and `[4..7]` bytes contain the ipv4 octets.
-    pub server_address: [u8; 28], // contains sockaddr_in or sockaddr_in6
-    pub map_id: u32,
-    pub map_type: u32,
-    pub shard_id: u32,
-    pub instance: u32,
-    pub build_id: u32,
-    // Additional data that gw2 provides us
-    pub ui_state: u32, // Bitmask: Bit 1 = IsMapOpen, Bit 2 = IsCompassTopRight, Bit 3 = DoesCompassHaveRotationEnabled, Bit 4 = Game has focus, Bit 5 = Is in Competitive game mode, Bit 6 = Textbox has focus, Bit 7 = Is in Combat
-    pub compass_width: u16, // pixels
-    pub compass_height: u16, // pixels
-    pub compass_rotation: f32, // radians
-    pub player_x: f32, // continentCoords
-    pub player_y: f32, // continentCoords
-    pub map_center_x: f32, // continentCoords
-    pub map_center_y: f32, // continentCoords
-    pub map_scale: f32,
-    pub process_id: u32,
-    pub mount_index: u8,
-}
-
-#[derive(Debug, Default, Clone, Serialize, Deserialize)]
-/// The json structure of the Identity field inside Cmumblelink.
-/// the json string is null terminated and utf-16 encoded. so, need to use
-/// Widestring crate's U16Cstring to first parse the bytes and then, convert to
-/// String before deserializing to CIdentity
-pub struct CIdentity {
-    pub name: String,
-    pub profession: u32,
-    pub spec: u32,
-    pub race: u32,
-    pub map_id: u32,
-    pub world_id: u32,
-    pub team_color_id: u32,
-    pub commander: bool,
-    pub fov: f32,
-    pub uisz: u32,
-}
-
 impl CMumbleLink {
     pub fn get_cmumble_link(link_ptr: *const CMumbleLink) -> CMumbleLink {
         unsafe { std::ptr::read_volatile(link_ptr) }
@@ -254,23 +235,3 @@ impl CMumbleLink {
         }
     }
 }
-
-// mod private {
-
-// #[derive(Debug, Default, Clone, Serialize, Deserialize)]
-// #[repr(C)]
-// pub struct MumbleLink {
-//     //pub ui_version: u32,
-//     pub ui_tick: u32,
-//     pub f_avatar_position: [f32; 3],
-//     pub f_avatar_front: [f32; 3],
-//     //pub f_avatar_top: [f32; 3],
-//     //pub name: [u16; 256],
-//     pub f_camera_position: [f32; 3],
-//     pub f_camera_front: [f32; 3],
-//     //pub f_camera_top: [f32; 3],
-//     pub identity: Identity,
-//     //pub context_len: u32,
-//     pub context: MumbleContext,
-// }
-// }
